unit Carddefs;

interface

uses Windows, SysUtils;

const Solitaire = 'Carddef.dll';
         Animation = 'Cardjkr.dll';
         FaceNames: array [1..13] of string = ('Ace', 'Two', 'Three', 'Four', 'Five', 'Six', 'Seven', 'Eight', 'Nine', 'Ten', 'Jack', 'Queen', 'King');
         SuitNames: array[1..4] of string = ('Clubs', 'Diamonds', 'Hearts', 'Spades');
          
type TCardSuit   = (csClubs, csDiamonds, csHearts, csSpades);
       TCardFace = (cdAce, cd2, cd3, cd4, cd5, cd6, cd7, cd8, cd9, cd10, cdJack, cdQueen, cdKing);
       TCardVar = record
          Suit: TCardSuit;
          Face: TCardFace;
       end;
       TCardIndex = 1..52;
       TCardBack = 1..9;

function CardVar(CdSuit: TCardSuit; CdFace: TCardFace): TCardVar; overload
function CardVar(Index: TCardIndex): TCardVar; overload
function GetCardFace(Index: TCardIndex): TCardFace;
function GetCardSuit(Index: TCardIndex): TCardSuit;
function GetCardIndex(CardVar: TCardVar): TCardIndex;
function LoadImage(CardVar: TCardVar): HBITMAP; overload
function LoadImage(Index: Integer): HBITMAP; overload
function LoadCardBack(Back: TCardBack): HBITMAP;
function LoadPreset(DBDLL: PChar): boolean;
function FreePreset: boolean;

implementation

var Handle: cardinal = 0;

function GetCardFace(Index: TCardIndex): TCardFace;
const CardFaces: array [1..13] of TCardFace = (cdAce, cd2, cd3, cd4, cd5, cd6, cd7, cd8, cd9, cd10, cdJack, cdQueen, cdKing);
var Actual: integer;
begin
  Actual:= 0;
  {we get a card index}
  if Index <= 13 then Actual:= Index;
  if (Index > 13) and (Index <= 26) then Actual:= Index - 13;
  if (Index > 26) and (Index <= 39) then Actual:= Index - 26;
  {we do not need to check the value because it is the last suit}
  if (Index > 39) then Actual:= Index - 39;
  Result:= CardFaces[Actual];
end;

function GetCardSuit(Index: TCardIndex): TCardSuit;
begin
  if (Index <= 13) then Result:= csClubs;
  if (Index > 13) and (Index <= 26) then Result:= csDiamonds;
  if (Index > 26) and (Index <= 39) then Result:= csHearts;
  if (Index > 39) then Result:= csSpades;
end;

function CardVar(CdSuit: TCardSuit; CdFace: TCardFace): TCardVar; overload
begin
   with Result do
   begin
      Suit:= CdSuit;
      Face:= CdFace;
   end;
end;

function CardVar(Index: TCardIndex): TCardVar; overload
begin
   with Result do
   begin
      Suit:= GetCardSuit(Index);
      Face:= GetCardFace(Index);
   end;
end;

function GetCardIndex(CardVar: TCardVar): TCardIndex;
const SuitValues: array [TCardSuit] of cardinal = (0, 13, 26, 39);
         FaceValues: array [TCardFace] of cardinal = (1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13);
begin
   Result:= FaceValues[CardVar.Face] + SuitValues[CardVar.Suit];
end;

function LoadImage(CardVar: TCardVar): HBITMAP; overload;
begin
   Result:= 0;
   if Handle <= 32 then Exit;
   Result:= LoadBitmap(Handle, MakeIntResource(GetCardIndex(CardVar)));
   if Result = 0 then MessageBox(0,PChar(SysErrorMessage(GetLastError)+#13+'Resource Index: '+IntToStr(GetCardIndex(CardVar))),'Cards RunTime',16);
end;

function LoadImage(Index: Integer): HBITMAP;
begin
   Result:= 0;
   if Handle <= 32 then Exit;
   Result:= LoadBitmap(Handle, MakeIntResource(Index));
   if Result = 0 then MessageBox(0,PChar(SysErrorMessage(GetLastError)+#13+'Resource Index: '+IntToStr(Index)),'CardDef RunTime',16);
end;

{Standard Backs: 1 - Biohazard; 2 - Pearl;, 3 - Firestorm; Max 9 Backs in DBDLL file}
{CardDef Backs: See in "Change Card Back"}
function LoadCardBack(Back: TCardBack): HBITMAP;
begin
   Result:= 0;
   if Handle <= 32 then Exit;
   Result:= LoadBitmap(Handle, MakeIntResource(60+Back));
   if Result = 0 then MessageBox(0,PChar(SysErrorMessage(GetLastError)),'CardDef Run-Time',16);
end;

function LoadPreset(DBDLL: PChar): boolean;
begin
  Handle:= LoadLibrary(DBDLL);
  Result:= Handle > 32;
end;

function FreePreset: boolean;
begin
  Result:= false;
  if (Handle <= 32) then Exit
  else Result:= FreeLibrary(Handle);
end;

end.
