unit GameSyst;
// Project: CardDef LAC
// Copyright  2006-07 DSoft
// Uses CardDef DLL. CardDef  2003 DSoft

// BIOS Part

// Please note that this unit does not perform some of the game functions
// (such as outscoring and penalizing) which are performed inside the program interface

interface

uses Classes, SysUtils;

// Messages may be translated to a different language if needed
const ER_GATE = 1;
      ER_SGATE = 2;
      ER_LGATE = 3;
      ER_FINISH = 4;
      ER_OK = 0;
      MER_GATE = 'You are in a gate. You need to score %d more points to add them';
      MER_SGATE = 'You must score at least %d more points to pass the starting gate';
      MER_LGATE = 'You may not have more than 940 points in the final gate';
      MER_FINISH = 'You have passed the final gate. You win!';
      MER_ZERO = 'Please throw cards at least once'; // used only within program. EDIT: Currently unused
      MER_OK = 'OK';
      STAT_1 = 'Starting gate (%d)';
      STAT_2 = 'Second gate (%d)';
      STAT_3 = 'Mid-way gate (%d)';
      STAT_4 = 'Final gate (%d)';
      STAT_5 = 'Free Scoring (%d)';
      // messages below are only used within program
      PENAL_1 = 'You scored zero. You got a minus';
      PENAL_2 = 'You scored three minuses. You lose 50 points';
      PENAL_3 = 'All outscored players lose 50 points'; // only if we do not display player names
      // only if a list box is used
      NAME2_FMT = '%s    |    %s    ';
      NAME3_FMT = '%s    |    %s    |    %s';
      NAME4_FMT = '%s    |    %s    |    %s    |    %s';
      SCOR2_FMT = '%d    |    %d';
      SCOR3_FMT = '%d    |    %d    |    %d';
      SCOR4_FMT = '%d    |    %d    |    %d    |    %d';

type string8 = string[8];
     TDies = array [1..5] of record
       Value: byte;
       Held: boolean;
     end;
     TTurn = record
       TurnValue: integer;
       Dies: TDies;
     end;
     TScores = array [1..4, 1..100] of integer;
     TNames = array [1..4] of string8;
     TIndex = array [1..4] of integer;
     TSavedGame = record
       PlayerNames: TNames;
       PlayerScores: TScores;
       ScoreIndex: TIndex;
       NumPlayers: byte;
       CurrentTurnIndex: integer;
       CurrentTurnValue: integer;
       CurrentDies: TDies;
       Minuses: TIndex;
     end;


procedure StartGame(NPlayers: byte; PNames: TNames);
procedure FinishGame;
function AddPoints(Value: integer; PlayerIndex: integer): integer;
function TurnValue(var Dies: TDies): integer;
procedure SaveGame(FileName: string; CurDies: TDies; CurTurnValue: integer);
function LoadGame(FileName: string; var Game: TSavedGame): boolean;
function PlayerScoringFormat(PlayerIndex: integer): string;
function Outscored(PlayerIndex1, PlayerIndex2, ScoreAdded: integer): boolean;
function CurrentScore(PlayerIndex: integer): integer;
procedure ClearDies(var Dies: TDies);
function CurPlayerIndex: integer;
procedure NextTurn;

var Scores: TScores;
    TurnIndex: integer;
    NumPlayers: byte;
    PlayerNames: TNames;
    ScoreIndex: TIndex;
    Minuses: TIndex;

implementation

procedure StartGame(NPlayers: byte; PNames: TNames);
var i, j: integer;
begin
 PlayerNames := PNames;
 NumPlayers := NPlayers;
 TurnIndex:= 1;
 for i:= 1 to 4 do
   for j:= 1 to 100 do
     Scores[i, j]:= 0;
  for i:= 1 to 4 do
    Minuses[i]:= 0;
end;

procedure FinishGame;
var i, j: integer;
begin
  for i:= 1 to NumPlayers do
   for j:= 1 to 100 do
     Scores[i, j]:= 0;
  for i:= 1 to 4 do
    Minuses[i]:= 0;
  for i:= 1 to 4 do
    PlayerNames[i]:= '';
end;

function CurrentScore(PlayerIndex: integer): integer;
begin
  Result:= Scores[PlayerIndex, ScoreIndex[PlayerIndex]];
    if Result < 0 then begin
     Result:= 0;
     Scores[PlayerIndex, ScoreIndex[PlayerIndex]] := 0;
    end;
end;

procedure AddScore(PlayerIndex: integer; Value: integer);
begin
  if Value > 0 then Minuses[PlayerIndex]:= 0;
  ScoreIndex[PlayerIndex]:= ScoreIndex[PlayerIndex] + 1;
  if ScoreIndex[PlayerIndex] <> 0 then
    Scores[PlayerIndex, ScoreIndex[PlayerIndex]]:= Scores[PlayerIndex, ScoreIndex[PlayerIndex]-1] + Value
  else Scores[PlayerIndex, ScoreIndex[PlayerIndex]]:= Value;
end;

function Outscored(PlayerIndex1, PlayerIndex2, ScoreAdded: integer): boolean;
begin
  Result:= ((CurrentScore(PlayerIndex1) - ScoreAdded) < CurrentScore(PlayerIndex2)) and (CurrentScore(PlayerIndex1) > CurrentScore(PlayerIndex2));
end;

function GateScore(s: integer): boolean;
begin
   Result:= (s < 50) or ((s < 300) and (s >= 200)) or ((s < 700) and (s >= 600)) or ((s >= 900) and not (s > 1000));
end;


function InGate(PlayerIndex: integer): boolean;
begin
  Result:= GateScore(CurrentScore(PlayerIndex));
end;

function AddPoints(Value: integer; PlayerIndex: integer): integer;
begin
  Result:= ER_OK;
  if Value = -50 then begin
    AddScore(PlayerIndex, Value);
    Exit;
  end;
  if Value = 0 then begin
    if CurrentScore(PlayerIndex) < 50 then Exit;
    Minuses[PlayerIndex]:= Minuses[PlayerIndex] + 1;
    Exit;
  end;
  if CurrentScore(PlayerIndex)+Value >= 1000 then
  begin
    AddScore(PlayerIndex, Value);
    Result:= ER_FINISH;
    Exit;
  end;
//  if CurrentScore(PlayerIndex) >= 1000 then begin Result:= ER_FINISH; Exit; end
  if InGate(PlayerIndex) then
  begin
    if not (CurrentScore(PlayerIndex)+Value >= 1000) then
    if (CurrentScore(PlayerIndex)+Value > 940) then begin Result:= ER_LGATE; Exit; end;
    if (CurrentScore(PlayerIndex) > 50)  then begin
     if GateScore(CurrentScore(PlayerIndex)+Value) and (CurrentScore(PlayerIndex)+Value <> 940) then begin Result:= ER_GATE; Exit; end;
    end
    else if GateScore(CurrentScore(PlayerIndex)+Value) then begin Result:= ER_SGATE; Exit; end;
  end;
  AddScore(PlayerIndex, Value);
  if CurrentScore(PlayerIndex) >= 1000 then begin Result:= ER_FINISH; Exit; end
end;

function PlayerScoringFormat(PlayerIndex: integer): string;
begin
  if InGate(PlayerIndex) then
  begin
    if (CurrentScore(PlayerIndex) < 50) then Result:= STAT_1;
    if (CurrentScore(PlayerIndex) >= 200) and (CurrentScore(PlayerIndex) < 300) then
      Result:= STAT_2;
    if (CurrentScore(PlayerIndex) >= 600) and (CurrentScore(PlayerIndex) < 700) then
      Result:= STAT_3;
    if (CurrentScore(PlayerIndex) >= 900) and (CurrentScore(PlayerIndex) < 1000) then
      Result:= STAT_4;
  end
  else Result:= STAT_5;
end;

function TurnValue(var Dies: TDies): integer;

// 3 of a kind or more values
// held state is set here for all "x Of A Kind"s
function OfAKindValue: integer;
var s1, s2, s3, s4, s5, s6: integer;
    i, j, v: integer;
begin
  s1:= 0;
  s2:= 0;
  s3:= 0;
  s4:= 0;
  s5:= 0;
  s6:= 0;
  v:= 0;
  for i:= 1 to 5 do
  begin
    if not Dies[i].Held then
    case Dies[i].Value of
     1: s1:= s1 + 1;
     2: s2:= s2 + 1;
     3: s3:= s3 + 1;
     4: s4:= s4 + 1;
     5: s5:= s5 + 1;
     6: s6:= s6 + 1;
    end;
  end;
  if (s1 >= 3) then begin
    v:= (100*(s1-2));
    for j:= 1 to 5 do
      if Dies[j].Value = 1 then Dies[j].Held:= true;
  end;
  if (s2 >= 3) then begin
    v:= (20*(s2-2));
    for j:= 1 to 5 do
      if Dies[j].Value = 2 then Dies[j].Held:= true;
  end;
  if (s3 >= 3) then begin
    v:= (30*(s3-2));
    for j:= 1 to 5 do
      if Dies[j].Value = 3 then Dies[j].Held:= true;
  end;
  if (s4 >= 3) then begin
    v:= (40*(s4-2));
    for j:= 1 to 5 do
      if Dies[j].Value = 4 then Dies[j].Held:= true;
  end;
  if (s5 >= 3) then begin
    v:= (50*(s5-2));
    for j:= 1 to 5 do
      if Dies[j].Value = 5 then Dies[j].Held:= true;
  end;
  if (s6 >= 3) then begin
    v:= (60*(s6-2));
    for j:= 1 to 5 do
      if Dies[j].Value = 6 then Dies[j].Held:= true;
  end;
  Result:= v;
end;

// values of free 5's and 1's
function SingleDiesValue: integer;
var i, v: integer;
begin
  v:= 0;
  for i:= 1 to 5 do
  begin
    if (Dies[i].Value = 1) and (not Dies[i].Held) then
    begin
      v:= v + 10;
      Dies[i].Held:= true;
    end;
    if (Dies[i].Value = 5) and (not Dies[i].Held) then
    begin
      v:= v + 5;
      Dies[i].Held:= true;
    end;
  end;
  Result:= v;
end;

var s1, s2: integer;

begin
  // a little comment needed here
  // we need a OfAKindValue calculated BEFORE SingleDiesValue to avoid errors
  // when held state is not set correctly for 5's and 1's
  // if we would just write OfAKindValue + SingleDiesValue, functions
  // may be executed in the incorrect order (see game rules for more info)
  s1:= OfAKindValue;
  s2:= SingleDiesValue;
  TurnValue:= s1 + s2;
end;

procedure SaveGame(FileName: string; CurDies: TDies; CurTurnValue: integer);
var CGame: TSavedGame; S: TStream;
begin
  CGame.PlayerNames:= PlayerNames;
  CGame.PlayerScores:= Scores;
  CGame.NumPlayers:= NumPlayers;
  CGame.CurrentTurnIndex:= TurnIndex;
  CGame.CurrentTurnValue:= CurTurnValue;
  CGame.CurrentDies:= CurDies;
  CGame.Minuses:= Minuses;
  CGame.ScoreIndex:= ScoreIndex;
  S:= TFileStream.Create(FileName, fmCreate);
  S.Write(CGame, SizeOf(CGame));
  S.Free;
end;

procedure NextTurn;
var i: integer;
begin
  if (TurnIndex = NumPlayers) then TurnIndex:= 1
  else TurnIndex:= TurnIndex + 1;
end;

procedure ClearDies(var Dies: TDies);
var i: integer;
begin
  for i:= 1 to 5 do
    Dies[i].Held:= false;
end;

function LoadGame(FileName: string; var Game: TSavedGame): boolean;
var s: TStream;
begin
  Result:= true;
  try
    S:= TFileStream.Create(FileName, fmOpenRead);
    S.ReadBuffer(Game, SizeOf(Game));
  except
    Result:= false;
    Exit;
  end;
  S.Free;
  Scores:= Game.PlayerScores;
  PlayerNames:= Game.PlayerNames;
  NumPlayers:= Game.NumPlayers;
  TurnIndex:= Game.CurrentTurnIndex;
  ScoreIndex:= Game.ScoreIndex;
  Minuses:= Game.Minuses;
end;

function CurPlayerIndex: integer;
begin
  Result:= TurnIndex;
end;

end.
