unit OptionsWin;
// Project: CardDef LAC
// Copyright  2006-07 DSoft
// Uses CardDef DLL. CardDef  2003 DSoft

// Options Window
// This unit is used by the interface to control game options

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, CardSWin, BackSWin, IniFiles, CardBack;

const CfgFileName = 'LAC.INI';

type
  TOptionsFrm = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Label1: TLabel;
    ProgBackBtn: TButton;
    StdProgBack: TCheckBox;
    Label2: TLabel;
    ScoreBackBtn: TButton;
    StdScoreBack: TCheckBox;
    Label3: TLabel;
    Label4: TLabel;
    CardBackBtn: TButton;
    StdCardBack: TCheckBox;
    Label5: TLabel;
    PlScorBackBtn: TButton;
    StdPlScoreBack: TCheckBox;
    Panel3: TPanel;
    Label6: TLabel;
    CardBacksBtn: TButton;
    OkBtn: TButton;
    ChooseCardsBtn: TButton;
    Label7: TLabel;
    Panel4: TPanel;
    CancelBtn: TButton;
    UseSoundEffects: TCheckBox;
    UseStdBackgrounds: TCheckBox;
    UseStdCards: TCheckBox;
    UseCardAnims: TCheckBox;
    procedure StdProgBackClick(Sender: TObject);
    procedure StdScoreBackClick(Sender: TObject);
    procedure StdCardBackClick(Sender: TObject);
    procedure StdPlScoreBackClick(Sender: TObject);
    procedure UseStdBackgroundsClick(Sender: TObject);
    procedure UseStdCardsClick(Sender: TObject);
    procedure ProgBackBtnClick(Sender: TObject);
    procedure ScoreBackBtnClick(Sender: TObject);
    procedure CardBackBtnClick(Sender: TObject);
    procedure PlScorBackBtnClick(Sender: TObject);
    procedure CardBacksBtnClick(Sender: TObject);
    procedure ChooseCardsBtnClick(Sender: TObject);
    procedure OkBtnClick(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
  private
  public
    ProgBackFileName: string;
    CardBackFileName: string;
    PlayBackFileName: string;
    ProgBackColor: TColor;
    ScorBackColor1: TColor;
    ScorBackColor2: TColor;
    PlayBackColor: TColor;
    CardBackColor: TColor;
    EffectsEnabled: boolean;
    CardAnimsEnabled: boolean;
    StandardCards: boolean;
    StandardBacks: boolean;
    CustomCardSet: TCardSet;
    ProgBackMode: byte;
    CardBackMode: byte;
    PlayBackMode: byte;
    CardBackIndex: integer;
    procedure UpdateConfiguration;
  end;

var
  OptionsFrm: TOptionsFrm;

implementation

{$R *.DFM}

procedure TOptionsFrm.StdProgBackClick(Sender: TObject);
begin
  ProgBackBtn.Enabled:= not StdProgBack.Checked;
  if StdProgBack.Checked then ProgBackColor:= clYellow;
end;

procedure TOptionsFrm.StdScoreBackClick(Sender: TObject);
begin
  ScoreBackBtn.Enabled:= not StdScoreBack.Checked;
  if StdScoreBack.Checked then begin
    ScorBackColor1:= clMaroon;
    ScorBackColor2:= clMaroon;
  end;
end;

procedure TOptionsFrm.StdCardBackClick(Sender: TObject);
begin
  CardBackBtn.Enabled:= not StdCardBack.Checked;
  if StdCardBack.Checked then CardBackColor:= clGray;
end;

procedure TOptionsFrm.StdPlScoreBackClick(Sender: TObject);
begin
  PlScorBackBtn.Enabled:= not StdPlScoreBack.Checked;
  if StdPlScoreBack.Checked then PlayBackColor:= clTeal;
end;

procedure TOptionsFrm.UseStdBackgroundsClick(Sender: TObject);
begin
 if UseStdBackgrounds.Checked then
 begin
   StdPlScoreBack.Checked:= true;
   StdCardBack.Checked:= true;
   StdScoreBack.Checked:= true;
   StdProgBack.Checked:= true;
   Panel2.Enabled:= false;
   StdProgBackClick(Sender);
   StdScoreBackClick(Sender);
   StdCardBackClick(Sender);
   StdPlScoreBackClick(Sender);
  // StdPlScoreBackClick(Sender);
  // StdCardBackClick(Sender);
  // StdScoreBackClick(Sender);
  // StdProgBackClick(Sender);
 end
 else Panel2.Enabled:= true;
end;

procedure TOptionsFrm.UseStdCardsClick(Sender: TObject);
var i: integer;
begin
 ChooseCardsBtn.Enabled:= not UseStdCards.Checked;
 if UseStdCards.Checked then
  for i:= 1 to 6 do
   CustomCardSet[i]:= i;
end;

procedure TOptionsFrm.ProgBackBtnClick(Sender: TObject);
begin
  BackSelectFrm.BackColor:= ProgBackColor;
  BackSelectFrm.ShowModal;
  if not BackSelectFrm.Canceled then
  begin
    if BackSelectFrm.ReturnMode = 1 then
      ProgBackColor := BackSelectFrm.BackColor;
    if BackSelectFrm.ReturnMode = 2 then
      ProgBackFileName := BackSelectFrm.FileName;
    ProgBackMode:= BackSelectFrm.ReturnMode;
  end;
end;

procedure TOptionsFrm.ScoreBackBtnClick(Sender: TObject);
begin
  // due to the way the score field is drawn, image background cannot
  // be selected.
  BackSelectFrm.PictureBack.Enabled:= false;
  BackSelectFrm.BitmapBtn.Caption:= 'Choose Color 2...';
  BackSelectFrm.ShowModal;
  if not BackSelectFrm.Canceled then
  begin
    // Return does not matter. Score background will always return 1
    ScorBackColor1:= BackSelectFrm.BackColor;
    ScorBackColor2:= BackSelectFrm.BackColor2;
  end;
  BackSelectFrm.PictureBack.Enabled:= true;
  BackSelectFrm.BitmapBtn.Caption:= 'Choose Image...';
end;

procedure TOptionsFrm.CardBackBtnClick(Sender: TObject);
begin
  BackSelectFrm.ShowModal;
  if not BackSelectFrm.Canceled then
  begin
    if BackSelectFrm.ReturnMode = 1 then
      CardBackColor := BackSelectFrm.BackColor;
    if BackSelectFrm.ReturnMode = 2 then
      CardBackFileName := BackSelectFrm.FileName;
    CardBackMode:= BackSelectFrm.ReturnMode;
  end;
end;

procedure TOptionsFrm.PlScorBackBtnClick(Sender: TObject);
begin
  BackSelectFrm.ShowModal;
  if not BackSelectFrm.Canceled then
  begin
    if BackSelectFrm.ReturnMode = 1 then
      PlayBackColor := BackSelectFrm.BackColor;
    if BackSelectFrm.ReturnMode = 2 then
      PlayBackFileName := BackSelectFrm.FileName;
    PlayBackMode:= BackSelectFrm.ReturnMode;
  end;
end;

procedure TOptionsFrm.CardBacksBtnClick(Sender: TObject);
begin
  Form2.ShowModal;
  CardBackIndex:= CardBack.BackIndex;
end;

procedure TOptionsFrm.ChooseCardsBtnClick(Sender: TObject);
begin
  CardSelectFrm.ShowModal;
  CustomCardSet:= CardSelectFrm.CardSet;
end;

procedure TOptionsFrm.OkBtnClick(Sender: TObject);
begin
  EffectsEnabled:= UseSoundEffects.Checked;
  CardAnimsEnabled:= UseCardAnims.Checked;
  StandardCards:= UseStdCards.Checked;
  StandardBacks:= UseStdBackgrounds.Checked;
  UpdateConfiguration;
  CancelBtnClick(Sender); // load the new configuration (just to save little bit of code, we simulate cancel button click)
end;

procedure TOptionsFrm.UpdateConfiguration;
var IniFile: TMemIniFile; i: integer;
begin
  IniFile:= TMemIniFile.Create(CfgFileName);
  IniFile.WriteBool('Options', 'SoundEffects', UseSoundEffects.Checked);
  IniFile.WriteBool('Options', 'StdProgBack', StdProgBack.Checked);
  IniFile.WriteBool('Options', 'StdCardBack', StdCardBack.Checked);
  IniFile.WriteBool('Options', 'StdPlScoreBack', StdPlScoreBack.Checked);
  IniFile.WriteBool('Options', 'StdScoreBack',   StdScoreBack.Checked);
  IniFile.WriteBool('Options', 'StdCards', UseStdCards.Checked);
  IniFile.Writebool('Options', 'CardAnims', UseCardAnims.Checked);
  for i:= 1 to 6 do
    IniFile.WriteInteger('Cards', IntToStr(i), CustomCardSet[i]);
  IniFile.WriteInteger('Cards', 'B', CardBackIndex);
  IniFile.WriteInteger('Backs', 'ProgBack', ProgBackMode);
  IniFile.WriteInteger('Backs', 'CardBack', CardBackMode);
  IniFile.WriteInteger('Backs', 'PlScoreBack', PlayBackMode);
  IniFile.WriteInteger('Colors', 'ProgBack', ProgBackColor);
  IniFile.WriteInteger('Colors', 'ScoreBack1', ScorBackColor1);
  IniFile.WriteInteger('Colors', 'ScoreBack2', ScorBackColor2);
  IniFile.WriteString('Files', 'ProgBack', ProgBackFileName);
  IniFile.WriteString('Files', 'CardBack', CardBackFileName);
  IniFile.WriteString('Files', 'PlScorBack', PlayBackFileName);
  IniFile.UpdateFile;
  IniFile.Free;
end;

procedure TOptionsFrm.CancelBtnClick(Sender: TObject);
var IniFile: TMemIniFile;
    i: integer;
begin
  // Revert to old configuration
  IniFile:= TMemIniFile.Create(CfgFileName);
  UseSoundEffects.Checked:= IniFile.ReadBool('Options', 'SoundEffects', true);
  EffectsEnabled:= UseSoundEffects.Checked;
  StdProgBack.Checked:= IniFile.ReadBool('Options', 'StdProgBack', true);
  StdCardBack.Checked:= IniFile.ReadBool('Options', 'StdCardBack', true);
  StdPlScoreBack.Checked:= IniFile.ReadBool('Options', 'StdPlScoreBack', true);
  UseStdCards.Checked:= IniFile.ReadBool('Options', 'StdCards', true);
  UseCardAnims.Checked:= IniFile.ReadBool('Options', 'CardAnims', true);
  StdScoreBack.Checked:= IniFile.ReadBool('Options', 'StdScoreBack', true);
  for i:= 1 to 6 do
   CustomCardSet[i]:= IniFile.ReadInteger('Cards', IntToStr(i), i);
  CardBackIndex:= IniFile.ReadInteger('Cards', 'B', 63);
  ProgBackMode:= IniFile.ReadInteger('Backs', 'ProgBack', 1);
  PlayBackMode:= IniFile.ReadInteger('Backs', 'PlScorBack', 1);
  ProgBackColor:= IniFile.ReadInteger('Colors','ProgBack', clYellow);
  ScorBackColor1:= IniFile.ReadInteger('Colors', 'ScoreBack1', clOlive);
  ScorBackColor2:= IniFile.ReadInteger('Colors', 'ScoreBack2', clOlive);
  CardBackColor:= IniFile.ReadInteger('Colors', 'CardBack', clGray);
  PlayBackColor:= IniFile.ReadInteger('Colors', 'PlScorBack', clTeal);
  ProgBackFileName:= IniFile.ReadString('Files', 'ProgBack', '');
  CardBackFileName:= IniFile.ReadString('Files', 'CardBack', '');
  PlayBackFileName:= IniFile.ReadString('Files', 'PlScorBack', '');
  IniFile.Free;
  OptionsFrm.Close;
end;

end.
